function [U,R] = URfact(A,r)

% Implements UR factorization algorithm from Theorem 4.1.
% Note that we are saving N-1 matrices here (i.e., the smaller
% U_k matrices in the last block are saved separately).
% This is for consistency with Theorem 4.3 where this 
% factorization is used.

% Check size of A
[Nr,Nc] = size(A);
if Nr == Nc
    N = Nr;
else
    disp('Matrix is not square')
    return
end

% Allocate variable for upper triangular matrix R
R = zeros(N);

% Allocate variable for matrices U_k. 
% For k=1:N-r the size of U_k is (r+1)x(r+1), 
% after which it decreases. If U_k is smaller
% than (r+1)x(r+1), then it is saved in the upper left
% (r+1)x(r+1) block of U(:,:,k).
U = zeros(r+1,r+1,N-1);

% Run algorithm
%
% Step 1
Y = A(1:r,:);

% Step 2 (here U_k has size (r+1)x(r+1))
for k = 1:N-r
    Delta = [Y(:,1); A(k+r,k)];
    [Utemp,Rtemp] = qr(Delta);
    U(:,:,k) = Utemp;
    x = Rtemp(1);
    Z = Utemp'*[Y(:,2:N-k+1); A(k+r,k+1:N)];
    X = Z(1,:);
    Y = Z(2:r+1,:);
    R(k,k) = x;
    R(k,k+1:N) = X;
end

% Step 3 (now size of U_k starts decreasing)
% Y is now Y_{N-r}
for k = N-r+1:N-1
    Delta = Y(:,1);
    [Utemp,Rtemp] = qr(Delta);
    U(1:N-k+1,1:N-k+1,k) = Utemp;
    x = Rtemp(1);
    Z = Utemp'*Y(:,2:N-k+1);
    X = Z(1,:);
    Y = Z(2:N-k+1,:);
    R(k,k) = x;
    R(k,k+1:N) = X;
end
R(N,N) = Y;

%%%%%% Check (optional) %%%%%
% B = R;
% for k = N-1:-1:N-r+1
%     sizeU = N-k+1;
%     B(k:N,:) = U(1:sizeU,1:sizeU,k)*B(k:N,:);
% end
% for k = N-r:-1:1
%     B(k:k+r,:) = U(1:r+1,1:r+1,k)*B(k:k+r,:);
% end
% disp(norm(A-B))
%%%%%% End check %%%%%

end